# Copyright (C) 2019 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import filecmp
import os
import shutil
import sys
import tempfile
from unittest import mock

import ddt
import zipfile

from tackerclient.common import exceptions
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.osc.v1.vnfpkgm import vnf_package
from tackerclient.tests.unit.osc import base
from tackerclient.tests.unit.osc.v1.fixture_data import client
from tackerclient.tests.unit.osc.v1 import vnf_package_fakes
from tackerclient.v1_0 import client as proxy_client


def _create_zip():
    vnf_package_path = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                        'sample_vnf_package')
    tmpdir = tempfile.mkdtemp()
    tmparchive = os.path.join(tmpdir, 'sample_vnf_package')
    zip_file = shutil.make_archive(tmparchive, 'zip', vnf_package_path)
    return zip_file, tmpdir


def _get_columns_vnf_package(action='list', vnf_package_obj=None):
    columns = []
    if action == 'update':
        if vnf_package_obj.get('userDefinedData'):
            columns.extend(['User Defined Data'])
        if vnf_package_obj.get('operationalState'):
            columns.extend(['Operational State'])
        return columns

    columns.extend(['ID', 'Onboarding State', 'Operational State',
                    'Usage State', 'User Defined Data', 'Links'])

    if action in ['show', 'create']:
        if vnf_package_obj and vnf_package_obj[
            'onboardingState'] == 'ONBOARDED':
            columns.extend(['VNFD ID',
                            'VNF Provider',
                            'VNF Software Version',
                            'VNFD Version',
                            'Software Images',
                            'VNF Product Name',
                            'Checksum',
                            'Additional Artifacts'])

    return columns


class TestVnfPackage(base.FixturedTestCase):
    client_fixture_class = client.ClientFixture

    def setUp(self):
        super(TestVnfPackage, self).setUp()
        self.url = client.TACKER_URL
        self.header = {'content-type': 'application/json'}
        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = self.cs
        self.app.client_manager.tackerclient = self.client_manager


@ddt.ddt
class TestCreateVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestCreateVnfPackage, self).setUp()
        self.create_vnf_package = vnf_package.CreateVnfPackage(
            self.app, self.app_args, cmd_name='vnf package create')

    @ddt.data((["--user-data", 'Test_key=Test_value'],
               [('user_data', {'Test_key': 'Test_value'})]),
              ([], []))
    @ddt.unpack
    def test_take_action(self, arglist, verifylist):
        # command param
        parsed_args = self.check_parser(self.create_vnf_package, arglist,
                                        verifylist)

        if arglist:
            json = vnf_package_fakes.vnf_package_obj(
                attrs={'userDefinedData': {'Test_key': 'Test_value'}})
        else:
            json = vnf_package_fakes.vnf_package_obj()
        self.requests_mock.register_uri(
            'POST', self.url + '/vnfpkgm/v1/vnf_packages',
            json=json, headers=self.header)

        columns, data = (self.create_vnf_package.take_action(parsed_args))
        self.assertCountEqual(_get_columns_vnf_package(), columns)
        headers, attributes = vnf_package._get_columns(json)
        self.assertListItemsEqual(vnf_package_fakes.get_vnf_package_data(
            json, columns=attributes), data)


@ddt.ddt
class TestListVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestListVnfPackage, self).setUp()
        self.list_vnf_package = vnf_package.ListVnfPackage(
            self.app, self.app_args, cmd_name='vnf package list')
        self._vnf_packages = self._get_vnf_packages()

    def _get_vnf_packages(self, onboarded_vnf_package=False):
        return vnf_package_fakes.create_vnf_packages(
            count=3, onboarded_vnf_package=onboarded_vnf_package)

    def get_list_columns(self, all_fields=False, exclude_fields=None,
                         extra_fields=None, exclude_default=False):

        columns = ['Id', 'Vnf Product Name', 'Onboarding State', 'Usage State',
                   'Operational State', 'Links']
        complex_columns = [
            'Checksum',
            'Software Images',
            'User Defined Data',
            'Additional Artifacts']
        simple_columns = ['Vnfd Version', 'Vnf Provider', 'Vnfd Id',
                          'Vnf Software Version']

        if extra_fields:
            columns.extend(extra_fields)

        if exclude_fields:
            columns.extend([field for field in complex_columns
                           if field not in exclude_fields])
        if all_fields:
            columns.extend(complex_columns)
            columns.extend(simple_columns)

        if exclude_default:
            columns.extend(simple_columns)

        return columns

    def _get_mock_response_for_list_vnf_packages(
            self, filter_attribute, json=None):
        self.requests_mock.register_uri(
            'GET', self.url + '/vnfpkgm/v1/vnf_packages?' + filter_attribute,
            json=json if json else self._get_vnf_packages(),
            headers=self.header)

    def test_take_action_default_fields(self):
        parsed_args = self.check_parser(self.list_vnf_package, [], [])
        self.requests_mock.register_uri(
            'GET', self.url + '/vnfpkgm/v1/vnf_packages',
            json=self._vnf_packages, headers=self.header)
        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(), long_listing=True)

        for vnf_package_obj in self._vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True))
        self.assertCountEqual(self.get_list_columns(), actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    @ddt.data('all_fields', 'exclude_default')
    def test_take_action(self, arg):
        parsed_args = self.check_parser(
            self.list_vnf_package,
            ["--" + arg, "--filter", '(eq,onboardingState,ONBOARDED)'],
            [(arg, True), ('filter', '(eq,onboardingState,ONBOARDED)')])
        vnf_packages = self._get_vnf_packages(onboarded_vnf_package=True)
        self._get_mock_response_for_list_vnf_packages(
            'filter=(eq,onboardingState,ONBOARDED)&' + arg, json=vnf_packages)

        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        kwargs = {arg: True}
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(**kwargs), long_listing=True)

        for vnf_package_obj in vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True, **kwargs))

        self.assertCountEqual(self.get_list_columns(**kwargs), actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    def test_take_action_with_exclude_fields(self):
        parsed_args = self.check_parser(
            self.list_vnf_package,
            ["--exclude_fields", 'softwareImages,checksum,'
                                 'userDefinedData,additionalArtifacts',
             "--filter", '(eq,onboardingState,ONBOARDED)'],
            [('exclude_fields', 'softwareImages,checksum,'
                                'userDefinedData,additionalArtifacts'),
             ('filter', '(eq,onboardingState,ONBOARDED)')])
        vnf_packages = self._get_vnf_packages(onboarded_vnf_package=True)
        updated_vnf_packages = {'vnf_packages': []}
        for vnf_pkg in vnf_packages['vnf_packages']:
            vnf_pkg.pop('softwareImages')
            vnf_pkg.pop('checksum')
            vnf_pkg.pop('userDefinedData')
            vnf_pkg.pop('additionalArtifacts')
            updated_vnf_packages['vnf_packages'].append(vnf_pkg)
        self._get_mock_response_for_list_vnf_packages(
            'filter=(eq,onboardingState,ONBOARDED)&'
            'exclude_fields=softwareImages,checksum,'
            'userDefinedData,additionalArtifacts',
            json=updated_vnf_packages)

        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(
                exclude_fields=['softwareImages', 'checksum',
                                'userDefinedData', 'additionalArtifacts']),
            long_listing=True)

        for vnf_package_obj in updated_vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True))
        expected_columns = self.get_list_columns(
            exclude_fields=['Software Images', 'Checksum',
                            'User Defined Data', 'Additional Artifacts'])
        self.assertCountEqual(expected_columns, actual_columns)
        self.assertListItemsEqual(expected_data, list(data))

    @ddt.data((['--all_fields', '--fields', 'softwareImages'],
               [('all_fields', True), ('fields', 'softwareImages')]),
              (['--all_fields', '--exclude_fields', 'checksum'],
               [('all_fields', True), ('exclude_fields', 'checksum')]),
              (['--fields', 'softwareImages', '--exclude_fields', 'checksum'],
               [('fields', 'softwareImages'), ('exclude_fields', 'checksum')]))
    @ddt.unpack
    def test_take_action_with_invalid_combination(self, arglist, verifylist):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.list_vnf_package, arglist, verifylist)

    def test_take_action_with_valid_combination(self):
        parsed_args = self.check_parser(
            self.list_vnf_package,
            ["--fields", 'softwareImages,checksum', "--exclude_default"],
            [('fields', 'softwareImages,checksum'), ('exclude_default', True)])
        vnf_packages = self._get_vnf_packages(onboarded_vnf_package=True)
        updated_vnf_packages = {'vnf_packages': []}
        for vnf_pkg in vnf_packages['vnf_packages']:
            vnf_pkg.pop('userDefinedData')
            updated_vnf_packages['vnf_packages'].append(vnf_pkg)

        self._get_mock_response_for_list_vnf_packages(
            'exclude_default&fields=softwareImages,checksum',
            json=updated_vnf_packages)

        actual_columns, data = self.list_vnf_package.take_action(parsed_args)
        expected_data = []
        headers, columns = tacker_osc_utils.get_column_definitions(
            self.list_vnf_package.get_attributes(
                extra_fields=['softwareImages', 'checksum'],
                exclude_default=True),
            long_listing=True)

        for vnf_package_obj in updated_vnf_packages['vnf_packages']:
            expected_data.append(vnf_package_fakes.get_vnf_package_data(
                vnf_package_obj, columns=columns, list_action=True,
                exclude_default=True))

        self.assertCountEqual(self.get_list_columns(
            extra_fields=['Software Images', 'Checksum'],
            exclude_default=True),
            actual_columns)
        self.assertListItemsEqual(expected_data, list(data))


@ddt.ddt
class TestShowVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestShowVnfPackage, self).setUp()
        self.show_vnf_package = vnf_package.ShowVnfPackage(
            self.app, self.app_args, cmd_name='vnf package show')

    @ddt.data(True, False)
    def test_take_action(self, onboarded):
        vnf_package_obj = vnf_package_fakes.vnf_package_obj(
            onboarded_state=onboarded)
        arglist = [vnf_package_obj['id']]
        verifylist = [('vnf_package', vnf_package_obj['id'])]
        parsed_args = self.check_parser(self.show_vnf_package, arglist,
                                        verifylist)
        url = self.url + '/vnfpkgm/v1/vnf_packages/' + vnf_package_obj['id']
        self.requests_mock.register_uri('GET', url, json=vnf_package_obj,
                                        headers=self.header)
        columns, data = (self.show_vnf_package.take_action(parsed_args))
        self.assertCountEqual(_get_columns_vnf_package(
            vnf_package_obj=vnf_package_obj, action='show'), columns)

        headers, attributes = vnf_package._get_columns(vnf_package_obj)
        self.assertListItemsEqual(
            vnf_package_fakes.get_vnf_package_data(vnf_package_obj,
                                                   columns=attributes), data)

    def test_show_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.show_vnf_package, [], [])


class TestDeleteVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestDeleteVnfPackage, self).setUp()
        self.delete_vnf_package = vnf_package.DeleteVnfPackage(
            self.app, self.app_args, cmd_name='vnf package delete')

        # The Vnf Package to delete
        self._vnf_package = vnf_package_fakes.create_vnf_packages(count=3)

    def _mock_request_url_for_delete(self, vnf_pkg_index):
        url = (self.url + '/vnfpkgm/v1/vnf_packages/' +
               self._vnf_package['vnf_packages'][vnf_pkg_index]['id'])

        json = self._vnf_package['vnf_packages'][vnf_pkg_index]

        self.requests_mock.register_uri('GET', url, json=json,
                                        headers=self.header)
        self.requests_mock.register_uri('DELETE', url,
                                        headers=self.header, json={})

    def test_delete_one_vnf_package(self):
        arglist = [self._vnf_package['vnf_packages'][0]['id']]
        verifylist = [('vnf-package', [self._vnf_package['vnf_packages']
                                       [0]['id']])]

        parsed_args = self.check_parser(self.delete_vnf_package, arglist,
                                        verifylist)

        self._mock_request_url_for_delete(0)
        result = self.delete_vnf_package.take_action(parsed_args)
        self.assertIsNone(result)

    def test_delete_multiple_vnf_package(self):
        arglist = []
        for vnf_pkg in self._vnf_package['vnf_packages']:
            arglist.append(vnf_pkg['id'])
        verifylist = [('vnf-package', arglist)]
        parsed_args = self.check_parser(self.delete_vnf_package, arglist,
                                        verifylist)
        for i in range(0, 3):
            self._mock_request_url_for_delete(i)

        result = self.delete_vnf_package.take_action(parsed_args)
        self.assertIsNone(result)

    def test_delete_multiple_vnf_package_exception(self):
        arglist = [
            self._vnf_package['vnf_packages'][0]['id'],
            'xxxx-yyyy-zzzz',
            self._vnf_package['vnf_packages'][1]['id'],
        ]
        verifylist = [
            ('vnf-package', arglist),
        ]
        parsed_args = self.check_parser(self.delete_vnf_package,
                                        arglist, verifylist)

        self._mock_request_url_for_delete(0)

        url = (self.url + '/vnfpkgm/v1/vnf_packages/' + 'xxxx-yyyy-zzzz')
        body = {"error": exceptions.NotFound('404')}
        self.requests_mock.register_uri('GET', url, body=body,
                                        status_code=404, headers=self.header)
        self._mock_request_url_for_delete(1)
        self.assertRaises(exceptions.CommandError,
                          self.delete_vnf_package.take_action,
                          parsed_args)


@ddt.ddt
class TestUploadVnfPackage(TestVnfPackage):

    # The new vnf package created.
    _vnf_package = vnf_package_fakes.vnf_package_obj(
        attrs={'userDefinedData': {'Test_key': 'Test_value'}})

    def setUp(self):
        super(TestUploadVnfPackage, self).setUp()
        self.upload_vnf_package = vnf_package.UploadVnfPackage(
            self.app, self.app_args, cmd_name='vnf package upload')

    def test_upload_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.upload_vnf_package, [], [])

    def _mock_request_url_for_upload(self, method, status_code=202, body={}):
        if method == 'PUT':
            self.header = {'content-type': 'application/zip'}
            url = (self.url + '/vnfpkgm/v1/vnf_packages/' +
                   self._vnf_package['id'] + '/package_content')
        else:
            url = (self.url + '/vnfpkgm/v1/vnf_packages/' +
                   self._vnf_package['id'] + '/package_content/'
                                             'upload_from_uri')

        self.requests_mock.register_uri(method, url, json=body,
                                        headers=self.header,
                                        status_code=status_code)

    def _get_arglist_and_verifylist(self, method, path):
        if method == 'path':
            arglist = [
                self._vnf_package['id'],
                "--path", path
            ]
            verifylist = [
                ('path', path),
                ('vnf_package', self._vnf_package['id'])
            ]
        else:
            arglist = [
                self._vnf_package['id'],
                "--url", "http://uri:8000/test.zip",
                "--user-name", "Test_username",
                "--password", "Test_passoword",
            ]
            verifylist = [
                ('url', "http://uri:8000/test.zip"),
                ('user_name', 'Test_username'),
                ('password', 'Test_passoword'),
                ('vnf_package', self._vnf_package['id'])
            ]
        return arglist, verifylist

    @ddt.data('path', 'url')
    def test_upload_vnf_package(self, method):
        path = None
        if method == 'path':
            zip_file, temp_dir = _create_zip()
            path = zip_file

        arglist, verifylist = self._get_arglist_and_verifylist(method,
                                                               path)
        parsed_args = self.check_parser(self.upload_vnf_package, arglist,
                                        verifylist)
        with mock.patch.object(proxy_client.ClientBase,
                               '_handle_fault_response') as m:
            if method == 'url':
                self._mock_request_url_for_upload('POST')
                self.upload_vnf_package.take_action(parsed_args)
            else:
                self._mock_request_url_for_upload('PUT')
                self.upload_vnf_package.take_action(parsed_args)
                # Delete temporary folder
                shutil.rmtree(temp_dir)
        # check no fault response is received
        self.assertNotCalled(m)

    def test_upload_vnf_package_with_conflict_error(self):
        # Scenario in which vnf package is already in on-boarded state
        zip_file, temp_dir = _create_zip()
        arglist, verifylist = self._get_arglist_and_verifylist('path',
                                                               zip_file)

        parsed_args = self.check_parser(self.upload_vnf_package, arglist,
                                        verifylist)

        body = {"conflictingRequest": {
            "message": "VNF Package " + self._vnf_package['id'] +
                       " onboarding state is not CREATED", "code": 409}}
        self._mock_request_url_for_upload('PUT', status_code=409, body=body)

        self.assertRaises(exceptions.TackerClientException,
                          self.upload_vnf_package.take_action, parsed_args)
        # Delete temporary folder
        shutil.rmtree(temp_dir)

    def test_upload_vnf_package_failed_with_404_not_found(self):
        # Scenario in which vnf package is not found
        zip_file, temp_dir = _create_zip()
        arglist = [
            'dumy-id',
            "--path", zip_file
        ]
        verifylist = [
            ('path', zip_file),
            ('vnf_package', 'dumy-id')
        ]

        parsed_args = self.check_parser(self.upload_vnf_package, arglist,
                                        verifylist)

        error_message = "Can not find requested vnf package: dummy-id"
        body = {"itemNotFound": {"message": error_message, "code": 404}}
        url = self.url + '/vnfpkgm/v1/vnf_packages/dumy-id/package_content'

        self.requests_mock.register_uri(
            'PUT', url, json=body,
            status_code=404)

        exception = self.assertRaises(
            exceptions.TackerClientException,
            self.upload_vnf_package.take_action, parsed_args)

        self.assertEqual(error_message, exception.message)
        # Delete temporary folder
        shutil.rmtree(temp_dir)


@ddt.ddt
class TestUpdateVnfPackage(TestVnfPackage):

    def setUp(self):
        super(TestUpdateVnfPackage, self).setUp()
        self.update_vnf_package = vnf_package.UpdateVnfPackage(
            self.app, self.app_args, cmd_name='vnf package update')

    @ddt.data((["--user-data", 'Test_key=Test_value',
                "--operational-state", 'DISABLED'],
               [('user_data', {'Test_key': 'Test_value'}),
                ('operational_state', 'DISABLED')]),
              (["--user-data", 'Test_key=Test_value'],
               [('user_data', {'Test_key': 'Test_value'})]),
              (["--operational-state", 'DISABLED'],
               [('operational_state', 'DISABLED')]))
    @ddt.unpack
    def test_take_action(self, arglist, verifylist):
        vnf_package_obj = vnf_package_fakes.vnf_package_obj(
            onboarded_state=True)
        arglist.append(vnf_package_obj['id'])
        verifylist.append(('vnf_package', vnf_package_obj['id']))

        parsed_args = self.check_parser(self.update_vnf_package, arglist,
                                        verifylist)
        url = os.path.join(self.url, 'vnfpkgm/v1/vnf_packages',
                           vnf_package_obj['id'])
        fake_response = vnf_package_fakes.get_fake_update_vnf_package_obj(
            arglist)
        self.requests_mock.register_uri('PATCH', url,
                                        json=fake_response,
                                        headers=self.header)

        columns, data = self.update_vnf_package.take_action(parsed_args)
        self.assertCountEqual(_get_columns_vnf_package(
            vnf_package_obj=fake_response, action='update'), columns)
        self.assertListItemsEqual(
            vnf_package_fakes.get_vnf_package_data(fake_response), data)

    def test_update_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.update_vnf_package, [], [])

    def test_update_without_user_data_and_operational_state(self):
        vnf_package_obj = vnf_package_fakes.vnf_package_obj(
            onboarded_state=True)
        arglist = [vnf_package_obj['id']]

        verifylist = [('vnf_package', vnf_package_obj['id'])]

        parsed_args = self.check_parser(self.update_vnf_package, arglist,
                                        verifylist)
        self.assertRaises(SystemExit, self.update_vnf_package.take_action,
                          parsed_args)


@ddt.ddt
class TestDownloadVnfPackage(TestVnfPackage):

    # The new vnf package created.
    _vnf_package = vnf_package_fakes.vnf_package_obj(
        attrs={'userDefinedData': {'Test_key': 'Test_value'}})

    def setUp(self):
        super(TestDownloadVnfPackage, self).setUp()
        self.download_vnf_package = vnf_package.DownloadVnfPackage(
            self.app, self.app_args, cmd_name='vnf package download')

    def test_download_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.download_vnf_package, [], [])

    def _mock_request_url_for_download_vnfd(self, content_type, vnfd_data):
        self.header = {'content-type': content_type}
        url = os.path.join(self.url, 'vnfpkgm/v1/vnf_packages',
                           self._vnf_package['id'], 'vnfd')

        if content_type == 'text/plain':
            self.requests_mock.register_uri('GET', url,
                                            headers=self.header,
                                            text=vnfd_data)
        else:
            self.requests_mock.register_uri('GET', url,
                                            headers=self.header,
                                            content=vnfd_data)

    def _get_arglist_and_verifylist(self, accept_type, file_name):
        arglist = [
            self._vnf_package['id'],
            '--vnfd',
            '--type', accept_type,
            '--file', file_name
        ]
        verifylist = [
            ('type', accept_type),
            ('vnfd', True),
            ('vnf_package', self._vnf_package['id']),
            ('file', file_name)
        ]
        return arglist, verifylist

    def test_download_vnfd_from_vnf_package_for_type_text_plain(self):
        test_file = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                     'sample_vnf_package/Definitions/'
                     'etsi_nfv_sol001_common_types.yaml')

        local_file = tempfile.NamedTemporaryFile(suffix='vnfd_data.yaml')
        vnfd_data = open(test_file, 'r').read()
        arglist, verifylist = self._get_arglist_and_verifylist(
            'text/plain', local_file.name)
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        self._mock_request_url_for_download_vnfd('text/plain', vnfd_data)
        self.download_vnf_package.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")

    @ddt.data('application/zip', 'both')
    def test_download_vnfd_from_vnf_package(self, accept_type):
        test_file, temp_dir = _create_zip()
        # File in which VNFD data will be stored.
        # For testing purpose we are creating temporary zip file.
        local_file = tempfile.NamedTemporaryFile(suffix='vnfd_data.zip')
        vnfd_data = open(test_file, 'rb').read()
        arglist, verifylist = self._get_arglist_and_verifylist(
            accept_type, local_file.name)
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        # When --type argument is set to 'both', then 'Accept' header in
        # request is set to 'text/plain,application/zip' now it is up to the
        # NFVO to choose the format to return for a single-file VNFD and for
        # a multi-file VNFD, a ZIP file shall be returned. Here we have taken
        # the example of multi-file vnfd hence its retuning zip file and
        # setting the 'Content-Type' as 'application/zip' in response header.
        self._mock_request_url_for_download_vnfd('application/zip', vnfd_data)
        self.download_vnf_package.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")
        self.assertTrue(self._check_valid_zip_file(local_file.name))
        shutil.rmtree(temp_dir)

    def _check_valid_zip_file(self, zip_file):
        with zipfile.ZipFile(zip_file) as zf:
            ret = zf.testzip()
        return False if ret else True

    @mock.patch('builtins.print')
    def test_download_vnfd_from_vnf_package_without_file_arg(self, mock_print):
        # --file argument is optional when --type is set to 'text/plain'.
        arglist = [
            self._vnf_package['id'],
            '--vnfd',
            '--type', 'text/plain',
        ]
        verifylist = [
            ('type', 'text/plain'),
            ('vnfd', True),
            ('vnf_package', self._vnf_package['id']),
        ]
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        test_file = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                     'sample_vnf_package/Definitions/'
                     'etsi_nfv_sol001_common_types.yaml')

        vnfd_data = open(test_file, 'r').read()
        self._mock_request_url_for_download_vnfd('text/plain', vnfd_data)
        self.download_vnf_package.take_action(parsed_args)
        mock_print.assert_called_once_with(vnfd_data)

    @ddt.data('application/zip', 'both')
    def test_download_vnfd_from_vnf_package_failed_with_no_file_arg(
            self, accept_type):
        arglist = [
            self._vnf_package['id'],
            '--vnfd',
            '--type', accept_type,
        ]
        verifylist = [
            ('type', accept_type),
            ('vnfd', True),
            ('vnf_package', self._vnf_package['id']),
        ]
        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)
        with mock.patch.object(sys.stdout, "isatty") as mock_isatty:
            mock_isatty.return_value = True
            self.assertRaises(SystemExit,
                              self.download_vnf_package.take_action,
                              parsed_args)

    def test_download_vnf_package(self):
        file_name = 'vnf_package_data.zip'
        test_file, temp_dir = _create_zip()

        # file in which VNF Package data will be stored.
        # for testing purpose we are creating temporary zip file.
        local_file = tempfile.NamedTemporaryFile(suffix=file_name)
        vnf_package_data = open(test_file, 'rb').read()

        arglist = [
            self._vnf_package['id'],
            '--file', local_file.name
        ]
        verifylist = [
            ('vnf_package', self._vnf_package['id']),
            ('file', local_file.name)
        ]

        parsed_args = self.check_parser(self.download_vnf_package, arglist,
                                        verifylist)

        url = os.path.join(self.url, '/vnfpkgm/v1/vnf_packages',
                           self._vnf_package['id'], 'package_content')

        self.requests_mock.register_uri(
            'GET', url, headers={'content-type': 'application/zip'},
            content=vnf_package_data)

        self.download_vnf_package.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")
        self.assertTrue(self._check_valid_zip_file(local_file.name))
        shutil.rmtree(temp_dir)


@ddt.ddt
class TestDownloadVnfPackageArtifact(TestVnfPackage):
    # The new vnf package created.
    _vnf_package = vnf_package_fakes.vnf_package_obj(
        attrs={'userDefinedData': {'Test_key': 'Test_value'}})

    def setUp(self):
        super(TestDownloadVnfPackageArtifact, self).setUp()
        self.download_vnf_package_artifacts = vnf_package.\
            DownloadVnfPackageArtifact(
                self.app, self.app_args,
                cmd_name='vnf package artifact download')

    def test_download_no_options(self):
        self.assertRaises(base.ParserException, self.check_parser,
                          self.download_vnf_package_artifacts, [], [])

    def _mock_request_url_for_download_artifacts(
        self, artifact_path, artifact_data):
        self.header = {'content-type': 'text/plain'}
        url = os.path.join(self.url, 'vnfpkgm/v1/vnf_packages',
                           self._vnf_package['id'], 'artifacts', artifact_path)
        self.requests_mock.register_uri('GET', url,
                                        headers=self.header,
                                        text=artifact_data)

    def _get_arglist_and_verifylist(self, localfile):
        arglist = [
            self._vnf_package['id'],
            localfile.name[1:],
            '--file', localfile.name
        ]
        verifylist = [
            ('vnf_package', self._vnf_package['id']),
            ('artifact_path', localfile.name[1:]),
            ('file', localfile.name)
        ]
        return arglist, verifylist

    def test_download_artifacts_from_vnf_package(self):
        test_file = ('./tackerclient/tests//unit/osc/v1/fixture_data/'
                     'sample_vnf_package_artifacts/Scripts/'
                     'install.sh')

        local_file = tempfile.NamedTemporaryFile(suffix='install.sh')
        artifact_data = open(test_file, 'r').read()
        arglist, verifylist = self._get_arglist_and_verifylist(
            local_file)
        parsed_args = self.check_parser(
            self.download_vnf_package_artifacts, arglist, verifylist)
        self._mock_request_url_for_download_artifacts(
            local_file.name[1:], artifact_data)
        self.download_vnf_package_artifacts.take_action(parsed_args)
        self.assertTrue(filecmp.cmp(test_file, local_file.name),
                        "Downloaded contents don't match test file")



# Copyright 2018 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command
from osc_lib import utils

from tackerclient.i18n import _
from tackerclient.osc import sdk_utils
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.tacker import v1_0 as tackerV10

_attr_map = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('resource_type', 'Resource Type', tacker_osc_utils.LIST_BOTH),
    ('resource_id', 'Resource ID', tacker_osc_utils.LIST_BOTH),
    ('resource_state', 'Resource State', tacker_osc_utils.LIST_BOTH),
    ('event_type', 'Event Type', tacker_osc_utils.LIST_BOTH),
    ('timestamp', 'Timestamp', tacker_osc_utils.LIST_BOTH),
    ('event_details', 'Event Details', tacker_osc_utils.LIST_LONG_ONLY),
)

_EVENT = "event"

events_path = '/events'


def _get_columns(item):
    column_map = {}
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ShowEvent(command.ShowOne):
    _description = _("Show event given the event id.")

    def get_parser(self, prog_name):
        parser = super(ShowEvent, self).get_parser(prog_name)
        parser.add_argument(
            _EVENT,
            metavar="ID",
            help=_("ID of event to display")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _EVENT, parsed_args.event)
        obj = client.show_event(obj_id)
        display_columns, columns = _get_columns(obj[_EVENT])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_EVENT]),
            columns,)
        return (display_columns, data)


class ListEvent(command.Lister):
    _description = _("List events of resources.")

    def get_parser(self, prog_name):
        parser = super(ListEvent, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            help=_("id of the event to look up."))
        parser.add_argument(
            '--resource-type',
            help=_("resource type of the events to look up."))
        parser.add_argument(
            '--resource-id',
            help=_("resource id of the events to look up."))
        parser.add_argument(
            '--resource-state',
            help=_("resource state of the events to look up."))
        parser.add_argument(
            '--event-type',
            help=_("event type of the events to look up."))
        parser.add_argument(
            '--long',
            action='store_true',
            help=_("List additional fields in output"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        _params = {}
        if parsed_args.id:
            _params['id'] = parsed_args.id
        if parsed_args.resource_id:
            _params['resource_id'] = parsed_args.resource_id
        if parsed_args.resource_state:
            _params['resource_state'] = parsed_args.resource_id
        if parsed_args.event_type:
            _params['event_type'] = parsed_args.event_type
        if parsed_args.resource_type:
            _params['resource_type'] = parsed_args.resource_type
        events = client.list('events', events_path, True, **_params)
        data = {}
        data['events'] = events['events']
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_EVENT + 's']))



#
# Copyright 2013 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
import yaml

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.tacker import v1_0 as tackerV10


_VNFD = "vnfd"


class ListVNFD(tackerV10.ListCommand):
    """List VNFD that belong to a given tenant."""

    resource = _VNFD
    list_columns = ['id', 'name', 'template_source', 'description']

    def get_parser(self, prog_name):
        parser = super(ListVNFD, self).get_parser(prog_name)
        parser.add_argument(
            '--template-source',
            help=_("List VNFD with specified template source. Available \
                   options are 'onboarded' (default), 'inline' or 'all'"),
            action='store',
            default='onboarded')
        return parser

    def args2search_opts(self, parsed_args):
        search_opts = super(ListVNFD, self).args2search_opts(parsed_args)
        template_source = parsed_args.template_source
        if parsed_args.template_source:
            search_opts.update({'template_source': template_source})
        return search_opts


class ShowVNFD(tackerV10.ShowCommand):
    """Show information of a given VNFD."""

    resource = _VNFD


class CreateVNFD(tackerV10.CreateCommand):
    """Create a VNFD."""

    resource = _VNFD
    remove_output_fields = ["attributes"]

    def add_known_arguments(self, parser):
        parser.add_argument('--vnfd-file', help=_('Specify VNFD file'))
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Set a name for the VNFD'))
        parser.add_argument(
            '--description',
            help=_('Set a description for the VNFD'))

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        vnfd = None
        if not parsed_args.vnfd_file:
            raise exceptions.InvalidInput(reason="Invalid input for vnfd file")
        with open(parsed_args.vnfd_file) as f:
            vnfd = f.read()
            try:
                vnfd = yaml.load(vnfd, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not vnfd:
                raise exceptions.InvalidInput(reason="vnfd file is empty")
            body[self.resource]['attributes'] = {'vnfd': vnfd}
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'description'])
        return body


class DeleteVNFD(tackerV10.DeleteCommand):
    """Delete given VNFD(s)."""
    resource = _VNFD


class ShowTemplateVNFD(tackerV10.ShowCommand):
    """Show template of a given VNFD."""

    resource = _VNFD

    def run(self, parsed_args):
        self.log.debug('run(%s)', parsed_args)
        template = None
        data = self.get_data(parsed_args)
        try:
            attributes_index = data[0].index('attributes')
            attributes_json = data[1][attributes_index]
            template = jsonutils.loads(attributes_json).get('vnfd', None)
        except (IndexError, TypeError, ValueError) as e:
            self.log.debug('Data handling error: %s', str(e))
        print(template or _('Unable to display VNFD template!'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from tackerclient.tacker.v1_0.nfvo import vnffgd
from tackerclient.tests.unit import test_cli10
from unittest.mock import mock_open
from unittest.mock import patch


class CLITestV10VmVNFFGDJSON(test_cli10.CLITestV10Base):
    _RESOURCE = 'vnffgd'
    _RESOURCES = 'vnffgds'

    def setUp(self):
        plurals = {'vnffgds': 'vnffgd'}
        super(CLITestV10VmVNFFGDJSON, self).setUp(plurals=plurals)

    @patch("tackerclient.tacker.v1_0.nfvo.vnffgd.open",
           side_effect=mock_open(read_data="vnffgd"),
           create=True)
    def test_create_vnffgd_all_params(self, mo):
        cmd = vnffgd.CreateVNFFGD(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'my-name'
        attr_key = 'vnffgd'
        attr_val = 'vnffgd'
        description = 'vnffgd description'
        args = [
            name,
            '--vnffgd-file', 'vnffgd_file',
            '--description', description,
        ]
        position_names = ['name', 'description']
        position_values = [name, description]
        extra_body = {
            'template': {attr_key: attr_val},
        }

        self._test_create_resource(self._RESOURCE, cmd, name, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body)

    @patch("tackerclient.tacker.v1_0.nfvo.vnffgd.open",
           side_effect=mock_open(read_data="vnffgd"),
           create=True)
    def test_create_vnffgd_with_mandatory_params(self, mo):
        cmd = vnffgd.CreateVNFFGD(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'my-name'
        attr_key = 'vnffgd'
        attr_val = 'vnffgd'
        args = [
            name,
            '--vnffgd-file', 'vnffgd_file',
        ]
        position_names = ['name']
        position_values = [name]
        extra_body = {
            'template': {attr_key: attr_val},
        }

        self._test_create_resource(self._RESOURCE, cmd, name, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body)

    def test_list_vnffgds(self):
        cmd = vnffgd.ListVNFFGD(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='onboarded')

    def test_list_inline_vnffgds(self):
        cmd = vnffgd.ListVNFFGD(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='inline')

    def test_list_all_vnffgds(self):
        cmd = vnffgd.ListVNFFGD(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='all')

    def test_list_vnffgds_pagenation(self):
        cmd = vnffgd.ListVNFFGD(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='onboarded')

    def test_show_vnffgd_id(self):
        cmd = vnffgd.ShowVNFFGD(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id, args,
                                 ['id'])

    def test_show_vnffgd_id_name(self):
        cmd = vnffgd.ShowVNFFGD(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_delete_vnffgd(self):
        cmd = vnffgd.DeleteVNFFGD(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(self._RESOURCE, cmd, my_id, args)



# Copyright 2018 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import yaml

from osc_lib.command import command
from osc_lib import utils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.osc import sdk_utils
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.tacker import v1_0 as tackerV10

_VNFFG = 'vnffg'    # VNF Forwarding Graph
_NFP = 'nfp'        # Network Forwarding Path
_SFC = 'sfc'        # Service Function Chain
_FC = 'classifier'  # Flow Classifier

nfps_path = '/nfps'
fcs_path = '/classifiers'
sfcs_path = '/sfcs'

DEFAULT_ERROR_REASON_LENGTH = 100

_attr_map_vnffg = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('ns_id', 'NS ID', tacker_osc_utils.LIST_BOTH),
    ('vnffgd_id', 'VNFFGD ID', tacker_osc_utils.LIST_BOTH),
    ('status', 'Status', tacker_osc_utils.LIST_BOTH),
    ('description', 'Description', tacker_osc_utils.LIST_LONG_ONLY),
)

_attr_map_nfp = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('status', 'Status', tacker_osc_utils.LIST_BOTH),
    ('vnffg_id', 'VNFFG ID', tacker_osc_utils.LIST_BOTH),
    ('path_id', 'Path ID', tacker_osc_utils.LIST_BOTH),
)

_attr_map_sfc = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('status', 'Status', tacker_osc_utils.LIST_BOTH),
    ('nfp_id', 'NFP ID', tacker_osc_utils.LIST_BOTH),
)

_attr_map_fc = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('status', 'Status', tacker_osc_utils.LIST_BOTH),
    ('nfp_id', 'NFP ID', tacker_osc_utils.LIST_BOTH),
    ('chain_id', 'Chain ID', tacker_osc_utils.LIST_BOTH),
)

_formatters = {
    'attributes': tacker_osc_utils.format_dict_with_indention,
    'match': tacker_osc_utils.format_dict_with_indention,
    'chain': tacker_osc_utils.format_dict_with_indention,
}


def _get_columns(item):
    column_map = {
        'tenant_id': 'project_id',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class CreateVNFFG(command.ShowOne):
    _description = _("Create a new VNFFG.")

    def get_parser(self, prog_name):
        parser = super(CreateVNFFG, self).get_parser(prog_name)
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Set a name for the VNFFG'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID'))
        vnffgd_group = parser.add_mutually_exclusive_group(required=True)
        vnffgd_group.add_argument(
            '--vnffgd-id',
            help=_('VNFFGD ID to use as template to create VNFFG'))
        vnffgd_group.add_argument(
            '--vnffgd-name',
            help=_('VNFFGD Name to use as template to create VNFFG'))
        vnffgd_group.add_argument(
            '--vnffgd-template',
            help=_('VNFFGD file to create VNFFG'))
        parser.add_argument(
            '--vnf-mapping',
            help=_('List of logical VNFD name to VNF instance name mapping. '
                   'Example: VNF1:my_vnf1,VNF2:my_vnf2'))
        parser.add_argument(
            '--symmetrical',
            action='store_true',
            default=False,
            help=_('Should a reverse path be created for the NFP '
                   '(True or False)'))
        parser.add_argument(
            '--param-file',
            help=_('YAML file with specific VNFFG parameters'))
        parser.add_argument(
            '--description',
            help=_('Set a description for the VNFFG'))
        return parser

    def args2body(self, parsed_args):
        body = {_VNFFG: {}}
        body[_VNFFG]['attributes'] = {}

        client = self.app.client_manager.tackerclient
        if parsed_args.vnf_mapping:
            _vnf_mapping = dict()
            _vnf_mappings = parsed_args.vnf_mapping.split(",")
            for mapping in _vnf_mappings:
                vnfd_name, vnf = mapping.split(":", 1)
                _vnf_mapping[vnfd_name] = \
                    tackerV10.find_resourceid_by_name_or_id(
                        client, 'vnf', vnf)
            parsed_args.vnf_mapping = _vnf_mapping

        if parsed_args.vnffgd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(
                client, 'vnffgd', parsed_args.vnffgd_name)
            parsed_args.vnffgd_id = _id
        elif parsed_args.vnffgd_template:
            with open(parsed_args.vnffgd_template) as f:
                template = f.read()
                try:
                    template = yaml.load(template, Loader=yaml.SafeLoader)
                except yaml.YAMLError as e:
                    raise exceptions.InvalidInput(reason=e)
                if not template:
                    raise exceptions.InvalidInput(
                        reason='The vnffgd file is empty')
                body[_VNFFG]['vnffgd_template'] = template

        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                param_yaml = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not param_yaml:
                raise exceptions.InvalidInput(
                    reason='The parameter file is empty')
            body[_VNFFG]['attributes'] = {'param_values': param_yaml}
        tackerV10.update_dict(parsed_args, body[_VNFFG],
                              ['tenant_id', 'name', 'vnffgd_id',
                               'symmetrical', 'vnf_mapping', 'description'])
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        vnffg = client.create_vnffg(self.args2body(parsed_args))
        display_columns, columns = _get_columns(vnffg[_VNFFG])
        data = utils.get_item_properties(
            sdk_utils.DictModel(vnffg[_VNFFG]),
            columns,
            formatters=_formatters)
        return (display_columns, data)


class DeleteVNFFG(command.Command):
    _description = _("Delete VNFFG(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteVNFFG, self).get_parser(prog_name)
        parser.add_argument(
            _VNFFG,
            metavar="<VNFFG>",
            nargs="+",
            help=_("VNFFG(s) to delete (name or ID)")
        )
        parser.add_argument(
            '--force',
            default=False,
            action='store_true',
            help=_('Force delete VNFFG'))
        return parser

    def args2body(self, parsed_args):
        body = dict()
        if parsed_args.force:
            body[_VNFFG] = dict()
            body[_VNFFG]['attributes'] = dict()
            body[_VNFFG]['attributes']['force'] = True
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        failure = False
        deleted_ids = []
        failed_items = {}
        body = self.args2body(parsed_args)
        for resource_id in parsed_args.vnffg:
            try:
                obj = tackerV10.find_resourceid_by_name_or_id(
                    client, _VNFFG, resource_id)
                client.delete_vnffg(obj, body)
                deleted_ids.append(resource_id)
            except Exception as e:
                failure = True
                failed_items[resource_id] = e
        if failure:
            msg = ''
            if deleted_ids:
                msg = (_('Successfully deleted %(resource)s(s):'
                         ' %(deleted_list)s') % {'deleted_list':
                                                 ', '.join(deleted_ids),
                                                 'resource': _VNFFG})
            err_msg = _("\n\nUnable to delete the below"
                        " %s(s):") % _VNFFG
            for failed_id, error in failed_items.items():
                err_msg += (_('\n Cannot delete %(failed_id)s: %(error)s')
                            % {'failed_id': failed_id,
                               'error': error})
            msg += err_msg
            raise exceptions.CommandError(message=msg)
        else:
            print((_('All specified %(resource)s(s) deleted successfully')
                   % {'resource': _VNFFG}))
        return


class UpdateVNFFG(command.ShowOne):
    _description = _("Update VNFFG.")

    def get_parser(self, prog_name):
        parser = super(UpdateVNFFG, self).get_parser(prog_name)
        parser.add_argument(
            _VNFFG,
            metavar="<VNFFG>",
            help=_('VNFFG to update (name or ID)'))
        parser.add_argument(
            '--vnffgd-template',
            help=_('VNFFGD file to update VNFFG'))
        parser.add_argument(
            '--vnf-mapping',
            help=_('List of logical VNFD name to VNF instance name mapping.  '
                   'Example: VNF1:my_vnf1,VNF2:my_vnf2'))
        parser.add_argument(
            '--symmetrical',
            action='store_true',
            default=False,
            help=_('Should a reverse path be created for the NFP'))
        parser.add_argument(
            '--description',
            help=_('Set a description for the VNFFG'))
        parser.add_argument(
            '--param-file',
            help=_('YAML file with specific VNFFG parameters'))
        return parser

    def args2body(self, parsed_args):
        body = {_VNFFG: {}}
        body[_VNFFG]['attributes'] = {}

        client = self.app.client_manager.tackerclient

        if parsed_args.vnf_mapping:
            _vnf_mapping = dict()
            _vnf_mappings = parsed_args.vnf_mapping.split(",")
            for mapping in _vnf_mappings:
                vnfd_name, vnf = mapping.split(":", 1)
                _vnf_mapping[vnfd_name] = \
                    tackerV10.find_resourceid_by_name_or_id(
                        client, 'vnf', vnf)
            parsed_args.vnf_mapping = _vnf_mapping

        if parsed_args.vnffgd_template:
            with open(parsed_args.vnffgd_template) as f:
                template = f.read()
                try:
                    template = yaml.load(
                        template, Loader=yaml.SafeLoader)
                except yaml.YAMLError as e:
                    raise exceptions.InvalidInput(reason=e)
                if not template:
                    raise exceptions.InvalidInput(
                        reason='The vnffgd file is empty')
                body[_VNFFG]['vnffgd_template'] = template

        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                param_yaml = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not param_yaml:
                raise exceptions.InvalidInput(
                    reason='The parameter file is empty')
            body[_VNFFG]['attributes'] = {'param_values': param_yaml}
        tackerV10.update_dict(parsed_args, body[_VNFFG],
                              ['vnf_mapping', 'symmetrical', 'description'])
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNFFG, parsed_args.vnffg)
        vnffg = client.update_vnffg(obj_id, self.args2body(parsed_args))
        display_columns, columns = _get_columns(vnffg[_VNFFG])
        data = utils.get_item_properties(
            sdk_utils.DictModel(vnffg[_VNFFG]),
            columns,
            formatters=_formatters)
        return (display_columns, data)


class ListVNFFG(command.Lister):
    _description = ("List VNFFG(s) that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListVNFFG, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            help=_('List additional fields in output')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        data = client.list_vnffgs()
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map_vnffg, long_listing=parsed_args.long)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_VNFFG + 's']))


class ShowVNFFG(command.ShowOne):
    _description = _("Display VNFFG details")

    def get_parser(self, prog_name):
        parser = super(ShowVNFFG, self).get_parser(prog_name)
        parser.add_argument(
            _VNFFG,
            metavar="<VNFFG>",
            help=_('VNFFG to display (name or ID)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNFFG, parsed_args.vnffg)
        obj = client.show_vnffg(obj_id)
        display_columns, columns = _get_columns(obj[_VNFFG])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_VNFFG]),
            columns,
            formatters=_formatters)
        return (display_columns, data)


class ListNFP(command.Lister):
    _description = ("List NFP(s) that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListNFP, self).get_parser(prog_name)
        parser.add_argument(
            '--vnffg-id',
            help=_('List NFP(s) with specific VNFFG ID'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        _params = {}
        if parsed_args.vnffg_id:
            _params['vnffg_id'] = parsed_args.vnffg_id
        nfps = client.list('nfps', nfps_path, True, **_params)
        for nfp in nfps['nfps']:
            error_reason = nfp.get('error_reason', None)
            if error_reason and \
                    len(error_reason) > DEFAULT_ERROR_REASON_LENGTH:
                nfp['error_reason'] = error_reason[
                    :DEFAULT_ERROR_REASON_LENGTH]
                nfp['error_reason'] += '...'
        data = {}
        data['nfps'] = nfps['nfps']
        data = client.list_nfps()
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map_nfp, long_listing=None)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_NFP + 's']))


class ShowNFP(command.ShowOne):
    _description = _("Display NFP details")

    def get_parser(self, prog_name):
        parser = super(ShowNFP, self).get_parser(prog_name)
        parser.add_argument(
            _NFP,
            metavar="<NFP>",
            help=_('NFP to display (name or ID)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _NFP, parsed_args.nfp)
        obj = client.show_nfp(obj_id)
        display_columns, columns = _get_columns(obj[_NFP])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_NFP]),
            columns)
        return (display_columns, data)


class ListFC(command.Lister):
    _description = ("List flow classifier(s) that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListFC, self).get_parser(prog_name)
        parser.add_argument(
            '--nfp-id',
            help=_('List flow classifier(s) with specific nfp id'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID or project ID'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        _params = {}
        if parsed_args.nfp_id:
            _params['nfp_id'] = parsed_args.nfp_id
        if parsed_args.tenant_id:
            _params['tenant_id'] = parsed_args.tenant_id
        classifiers = client.list('classifiers', fcs_path, True,
                                  **_params)
        for classifier in classifiers['classifiers']:
            error_reason = classifier.get('error_reason', None)
            if error_reason and \
                    len(error_reason) > DEFAULT_ERROR_REASON_LENGTH:
                classifier['error_reason'] = error_reason[
                    :DEFAULT_ERROR_REASON_LENGTH]
                classifier['error_reason'] += '...'
        data = {}
        data['classifiers'] = classifiers['classifiers']
        data = client.list_classifiers()
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map_fc, long_listing=None)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_FC + 's']))


class ShowFC(command.ShowOne):
    _description = _("Display flow classifier details")

    def get_parser(self, prog_name):
        parser = super(ShowFC, self).get_parser(prog_name)
        parser.add_argument(
            _FC,
            metavar="<Classifier ID>",
            help=_('Flow Classifier to display (name or ID)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _FC, parsed_args.classifier)
        obj = client.show_classifier(obj_id)
        display_columns, columns = _get_columns(obj[_FC])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_FC]),
            columns,
            formatters=_formatters)
        return (display_columns, data)


class ListSFC(command.Lister):
    _description = ("List SFC(s) that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListSFC, self).get_parser(prog_name)
        parser.add_argument(
            '--nfp-id',
            help=_('List SFC(s) with specific nfp id'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID or project ID'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        _params = {}
        if parsed_args.nfp_id:
            _params['nfp_id'] = parsed_args.nfp_id
        if parsed_args.tenant_id:
            _params['tenant_id'] = parsed_args.tenant_id
        sfcs = client.list('sfcs', sfcs_path, True, **_params)
        for chain in sfcs['sfcs']:
            error_reason = chain.get('error_reason', None)
            if error_reason and \
                    len(error_reason) > DEFAULT_ERROR_REASON_LENGTH:
                chain['error_reason'] = error_reason[
                    :DEFAULT_ERROR_REASON_LENGTH]
                chain['error_reason'] += '...'
        data = {}
        data['sfcs'] = sfcs['sfcs']
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map_sfc, long_listing=None)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_SFC + 's']))


class ShowSFC(command.ShowOne):
    _description = _("Display SFC details")

    def get_parser(self, prog_name):
        parser = super(ShowSFC, self).get_parser(prog_name)
        parser.add_argument(
            _SFC,
            metavar="<SFC>",
            help=_('SFC to display (name or ID)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _SFC, parsed_args.sfc)
        obj = client.show_sfc(obj_id)
        display_columns, columns = _get_columns(obj[_SFC])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_SFC]),
            columns,
            formatters=_formatters)
        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.tacker import v1_0 as tackerV10


_VNFFG = 'vnffg'
_NFP = 'nfp'
_SFC = 'sfc'
_FC = 'classifier'


class ListFC(tackerV10.ListCommand):
    """List FCs that belong to a given tenant."""

    resource = _FC
    list_columns = ['id', 'status', 'nfp_id', 'chain_id']

    def extend_list(self, data, parsed_args):
        """Update the list_columns list.

        This method update the list_columns list by adding the
        'name' column in case the retrieved FC list from the tacker
        server side contains the names of the FCs.
        """
        for item in data:
            if 'name' in item:
                self.list_columns.insert(1, 'name')
                break


class ShowFC(tackerV10.ShowCommand):
    """Show information of a given FC."""

    resource = _FC


class ListSFC(tackerV10.ListCommand):
    """List SFCs that belong to a given tenant."""

    resource = _SFC
    list_columns = ['id', 'status', 'nfp_id']


class ShowSFC(tackerV10.ShowCommand):
    """Show information of a given SFC."""

    resource = _SFC


class ListNFP(tackerV10.ListCommand):
    """List NFPs that belong to a given tenant."""

    resource = _NFP
    list_columns = ['id', 'name', 'status', 'vnffg_id', 'path_id']


class ShowNFP(tackerV10.ShowCommand):
    """Show information of a given NFP."""

    resource = _NFP


class ListVNFFG(tackerV10.ListCommand):
    """List VNFFGs that belong to a given tenant."""

    resource = _VNFFG
    list_columns = ['id', 'name', 'ns_id',
                    'description', 'status', 'vnffgd_id']


class ShowVNFFG(tackerV10.ShowCommand):
    """Show information of a given VNFFG."""

    resource = _VNFFG


class CreateVNFFG(tackerV10.CreateCommand):
    """Create a VNFFG."""

    resource = _VNFFG
    remove_output_fields = ["attributes"]

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Set a name for the VNFFG'))
        vnffgd_group = parser.add_mutually_exclusive_group(required=True)
        vnffgd_group.add_argument(
            '--vnffgd-id',
            help=_('VNFFGD ID to use as template to create VNFFG'))
        vnffgd_group.add_argument(
            '--vnffgd-name',
            help=_('VNFFGD Name to use as template to create VNFFG'))
        vnffgd_group.add_argument(
            '--vnffgd-template',
            help=_('VNFFGD file to create VNFFG'))
        parser.add_argument(
            '--vnf-mapping',
            help=_('List of logical VNFD name to VNF instance name mapping.  '
                   'Example: VNF1:my_vnf1,VNF2:my_vnf2'))
        parser.add_argument(
            '--symmetrical',
            action='store_true',
            default=False,
            help=_('Should a reverse path be created for the NFP'))
        parser.add_argument(
            '--param-file',
            help='Specify parameter yaml file'
        )

    def args2body(self, parsed_args):
        args = {'attributes': {}}
        body = {self.resource: args}

        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format

        if parsed_args.vnf_mapping:
            _vnf_mapping = dict()
            _vnf_mappings = parsed_args.vnf_mapping.split(",")
            for mapping in _vnf_mappings:
                vnfd_name, vnf = mapping.split(":", 1)
                _vnf_mapping[vnfd_name] = \
                    tackerV10.find_resourceid_by_name_or_id(
                        tacker_client, 'vnf', vnf)

            parsed_args.vnf_mapping = _vnf_mapping

        if parsed_args.vnffgd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vnffgd',
                                                          parsed_args.
                                                          vnffgd_name)
            parsed_args.vnffgd_id = _id
        elif parsed_args.vnffgd_template:
            with open(parsed_args.vnffgd_template) as f:
                template = f.read()
            try:
                args['vnffgd_template'] = yaml.load(
                    template, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not args['vnffgd_template']:
                raise exceptions.InvalidInput(
                    reason='The vnffgd file is empty')

        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                args['attributes']['param_values'] = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)

        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'vnffgd_id',
                               'symmetrical', 'vnf_mapping'])
        return body


class UpdateVNFFG(tackerV10.UpdateCommand):
    """Update a given VNFFG."""

    resource = _VNFFG

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--vnffgd-template',
            help=_('VNFFGD file to update VNFFG')
        )
        parser.add_argument(
            '--vnf-mapping',
            help=_('List of logical VNFD name to VNF instance name mapping.  '
                   'Example: VNF1:my_vnf1,VNF2:my_vnf2'))
        parser.add_argument(
            '--symmetrical',
            action='store_true',
            default=False,
            help=_('Should a reverse path be created for the NFP'))

    def args2body(self, parsed_args):
        args = {}
        body = {self.resource: args}

        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format

        if parsed_args.vnf_mapping:
            _vnf_mapping = dict()
            _vnf_mappings = parsed_args.vnf_mapping.split(",")
            for mapping in _vnf_mappings:
                vnfd_name, vnf = mapping.split(":", 1)
                _vnf_mapping[vnfd_name] = \
                    tackerV10.find_resourceid_by_name_or_id(
                        tacker_client, 'vnf', vnf)

            parsed_args.vnf_mapping = _vnf_mapping

        if parsed_args.vnffgd_template:
            with open(parsed_args.vnffgd_template) as f:
                template = f.read()
            try:
                args['vnffgd_template'] = yaml.load(
                    template, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not args['vnffgd_template']:
                raise exceptions.InvalidInput(
                    reason='The vnffgd template is empty')

        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'vnf_mapping', 'symmetrical'])
        return body


class DeleteVNFFG(tackerV10.DeleteCommand):
    """Delete a given VNFFG."""

    resource = _VNFFG
    remove_output_fields = ["attributes"]

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--force',
            default=False,
            action='store_true',
            help=_('Force delete VNFFG'))

    def args2body(self, parsed_args):
        body = dict()
        if parsed_args.force:
            body[self.resource] = dict()
            body[self.resource]['attributes'] = {'force': True}
        return body



# Copyright 2018 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import yaml

from osc_lib.command import command
from osc_lib import utils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.osc import sdk_utils
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.tacker import v1_0 as tackerV10

_attr_map = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('template_source', 'Template_Source',
     tacker_osc_utils.LIST_BOTH),
    ('description', 'Description', tacker_osc_utils.LIST_BOTH),
)

_NSD = 'nsd'

_formatters = {
    'attributes': tacker_osc_utils.format_dict_with_indention,
}


def _get_columns(item):
    column_map = {
        'tenant_id': 'project_id',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class CreateNSD(command.ShowOne):
    _description = _("Create a new NSD.")

    def get_parser(self, prog_name):
        parser = super(CreateNSD, self).get_parser(prog_name)
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name for NSD'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID or project ID'))
        parser.add_argument(
            '--nsd-file',
            required=True,
            help=_('YAML file with NSD parameters'))
        parser.add_argument(
            '--description',
            help=_('Set a description for the NSD'))
        return parser

    def args2body(self, parsed_args):
        body = {_NSD: {}}
        nsd = None
        if not parsed_args.nsd_file:
            raise exceptions.InvalidInput(reason="Invalid input for nsd file")
        with open(parsed_args.nsd_file) as f:
            nsd = f.read()
            try:
                nsd = yaml.load(nsd, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not nsd:
                raise exceptions.InvalidInput(reason="nsd file is empty")
            body[_NSD]['attributes'] = {'nsd': nsd}
        tackerV10.update_dict(parsed_args, body[_NSD],
                              ['tenant_id', 'name', 'description'])
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        nsd = client.create_nsd(self.args2body(parsed_args))
        display_columns, columns = _get_columns(nsd[_NSD])
        nsd[_NSD]['attributes']['nsd'] = yaml.load(
            nsd[_NSD]['attributes']['nsd'])
        data = utils.get_item_properties(
            sdk_utils.DictModel(nsd[_NSD]),
            columns, formatters=_formatters)
        return (display_columns, data)


class DeleteNSD(command.Command):
    _description = _("Delete NSD(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteNSD, self).get_parser(prog_name)
        parser.add_argument(
            _NSD,
            metavar="<NSD>",
            nargs="+",
            help=_("NSD(s) to delete (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        failure = False
        deleted_ids = []
        failed_items = {}
        for resource_id in parsed_args.nsd:
            try:
                obj = tackerV10.find_resourceid_by_name_or_id(
                    client, _NSD, resource_id)
                client.delete_nsd(obj)
                deleted_ids.append(resource_id)
            except Exception as e:
                failure = True
                failed_items[resource_id] = e
        if failure:
            msg = ''
            if deleted_ids:
                msg = (_('Successfully deleted %(resource)s(s):'
                         ' %(deleted_list)s') % {'deleted_list':
                                                 ', '.join(deleted_ids),
                                                 'resource': _NSD})
            err_msg = _("\n\nUnable to delete the below"
                        " %s(s):") % _NSD
            for failed_id, error in failed_items.items():
                err_msg += (_('\n Cannot delete %(failed_id)s: %(error)s')
                            % {'failed_id': failed_id,
                               'error': error})
            msg += err_msg
            raise exceptions.CommandError(message=msg)
        else:
            print((_('All specified %(resource)s(s) deleted successfully')
                   % {'resource': _NSD}))
        return


class ListNSD(command.Lister):
    _description = ("List (NSD)s that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListNSD, self).get_parser(prog_name)
        parser.add_argument(
            '--template-source',
            help=_("List NSD with specified template source. Available \
                   options are 'onboared' (default), 'inline' or 'all'"),
            action='store',
            default='onboarded')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        data = client.list_nsds()
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map, long_listing=None)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_NSD + 's']))


class ShowNSD(command.ShowOne):
    _description = _("Display NSD details")

    def get_parser(self, prog_name):
        parser = super(ShowNSD, self).get_parser(prog_name)
        parser.add_argument(
            _NSD,
            metavar="<NSD>",
            help=_("NSD to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _NSD, parsed_args.nsd)
        obj = client.show_nsd(obj_id)
        obj[_NSD]['attributes']['nsd'] = yaml.load(
            obj[_NSD]['attributes']['nsd'])
        display_columns, columns = _get_columns(obj[_NSD])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_NSD]),
            columns,
            formatters=_formatters)
        return (display_columns, data)


class ShowTemplateNSD(command.ShowOne):
    _description = _("Display NSD Template details")

    def get_parser(self, prog_name):
        parser = super(ShowTemplateNSD, self).get_parser(prog_name)
        parser.add_argument(
            _NSD,
            metavar="<NSD>",
            help=_("NSD to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _NSD, parsed_args.nsd)
        obj = client.show_nsd(obj_id)
        obj[_NSD]['attributes']['nsd'] = yaml.load(
            obj[_NSD]['attributes']['nsd'])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_NSD]),
            ('attributes',),
            formatters=_formatters)
        data = (data or _('Unable to display NSD template!'))
        return (('attributes',), data)



# Copyright 2016 Brocade Communications Systems Inc
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tackerclient.tacker import v1_0 as tackerV10

_EVENT = "event"


class ListEventsBase(tackerV10.ListCommand):
    """Base class for list command."""

    list_columns = ['id', 'resource_type', 'resource_id',
                    'resource_state', 'event_type',
                    'timestamp', 'event_details']

    def get_parser(self, prog_name):
        parser = super(ListEventsBase, self).get_parser(prog_name)
        parser.add_argument('--id',
                            help='id of the event to look up.')
        parser.add_argument('--resource-id',
                            help='resource id of the events to look up.')
        parser.add_argument('--resource-state',
                            help='resource state of the events to look up.')
        parser.add_argument('--event-type',
                            help='event type of the events to look up.')
        return parser

    def args2search_opts(self, parsed_args):
        search_opts = super(ListEventsBase, self).args2search_opts(
            parsed_args)
        if parsed_args.id:
            search_opts.update({'id': parsed_args.id})
        if parsed_args.resource_id:
            search_opts.update({'resource_id': parsed_args.resource_id})
        if parsed_args.resource_state:
            search_opts.update({'resource_state': parsed_args.resource_state})
        if parsed_args.event_type:
            search_opts.update({'event_type': parsed_args.event_type})
        return search_opts


class ListResourceEvents(ListEventsBase):
    """List events of resources."""

    resource = _EVENT

    def get_parser(self, prog_name):
        parser = super(ListResourceEvents, self).get_parser(prog_name)
        parser.add_argument('--resource-type',
                            help='resource type of the events to look up.')
        return parser

    def args2search_opts(self, parsed_args):
        search_opts = super(ListResourceEvents, self).args2search_opts(
            parsed_args)
        if parsed_args.resource_type:
            search_opts.update({'resource_type': parsed_args.resource_type})
        return search_opts


class ListVNFEvents(ListEventsBase):
    """List events of VNFs."""

    resource = "vnf_event"


class ListVNFDEvents(ListEventsBase):
    """List events of VNFDs."""

    resource = "vnfd_event"


class ListVIMEvents(ListEventsBase):
    """List events of VIMs."""

    resource = "vim_event"


class ShowEvent(tackerV10.ShowCommand):
    """Show event given the event id."""

    resource = _EVENT



#
# Copyright 2013 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import yaml

from oslo_utils import encodeutils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.tacker import v1_0 as tackerV10


_VNF = 'vnf'
_RESOURCE = 'resource'


class ListVNF(tackerV10.ListCommand):
    """List VNF that belong to a given tenant."""

    resource = _VNF
    list_columns = ['id', 'name', 'mgmt_ip_address', 'status',
                    'vim_id', 'vnfd_id']


class ShowVNF(tackerV10.ShowCommand):
    """Show information of a given VNF."""

    resource = _VNF


class CreateVNF(tackerV10.CreateCommand):
    """Create a VNF."""

    resource = _VNF
    remove_output_fields = ["attributes"]

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Set a name for the VNF'))
        parser.add_argument(
            '--description',
            help=_('Set description for the VNF'))
        vnfd_group = parser.add_mutually_exclusive_group(required=True)
        vnfd_group.add_argument(
            '--vnfd-id',
            help=_('VNFD ID to use as template to create VNF'))
        vnfd_group.add_argument(
            '--vnfd-name',
            help=_('VNFD Name to use as template to create VNF'))
        vnfd_group.add_argument(
            '--vnfd-template',
            help=_("VNFD file to create VNF"))
        vim_group = parser.add_mutually_exclusive_group()
        vim_group.add_argument(
            '--vim-id',
            help=_('VIM ID to use to create VNF on the specified VIM'))
        vim_group.add_argument(
            '--vim-name',
            help=_('VIM name to use to create VNF on the specified VIM'))
        parser.add_argument(
            '--vim-region-name',
            help=_('VIM Region to use to create VNF on the specified VIM'))
        parser.add_argument(
            '--config-file',
            help=_('YAML file with VNF configuration'))
        parser.add_argument(
            '--param-file',
            help=_('Specify parameter yaml file'))

    def args2body(self, parsed_args):
        args = {'attributes': {}}
        body = {self.resource: args}
        # config arg passed as data overrides config yaml when both args passed
        config = None
        if parsed_args.config_file:
            with open(parsed_args.config_file) as f:
                config_yaml = f.read()
            try:
                config = yaml.load(
                    config_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)

        if config:
            args['attributes']['config'] = config
        if parsed_args.vim_region_name:
            args.setdefault('placement_attr', {})['region_name'] = \
                parsed_args.vim_region_name

        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format
        if parsed_args.vim_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vim',
                                                          parsed_args.
                                                          vim_name)
            parsed_args.vim_id = _id
        if parsed_args.vnfd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vnfd',
                                                          parsed_args.
                                                          vnfd_name)
            parsed_args.vnfd_id = _id
        elif parsed_args.vnfd_template:
            with open(parsed_args.vnfd_template) as f:
                template = f.read()
            try:
                args['vnfd_template'] = yaml.load(
                    template, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)

        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                args['attributes']['param_values'] = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'description',
                               'vnfd_id', 'vim_id'])
        return body


class UpdateVNF(tackerV10.UpdateCommand):
    """Update a given VNF."""

    resource = _VNF

    def add_known_arguments(self, parser):
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            '--config-file',
            help=_('YAML file with VNF configuration'))
        group.add_argument(
            '--config',
            help=_('YAML data with VNF configuration'))
        group.add_argument(
            '--param-file',
            help=_('YAML file with VNF parameter'))

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        # config arg passed as data overrides config yaml when both args passed
        config = None
        if parsed_args.config_file:
            with open(parsed_args.config_file) as f:
                config_yaml = f.read()
            try:
                config = yaml.load(config_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not config:
                raise exceptions.InvalidInput(
                    reason='The config file is empty')
        if parsed_args.config:
            config_param = encodeutils.safe_decode(parsed_args.config)
            try:
                config = yaml.load(config_param, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not config:
                raise exceptions.InvalidInput(
                    reason='The parameter is empty')
        if config:
            body[self.resource]['attributes'] = {'config': config}
        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                param = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not param:
                raise exceptions.InvalidInput(
                    reason='The parameter file is empty')
            body[self.resource]['attributes'] = {'param_values': param}
        tackerV10.update_dict(parsed_args, body[self.resource], ['tenant_id'])
        return body


class DeleteVNF(tackerV10.DeleteCommand):
    """Delete given VNF(s)."""

    resource = _VNF
    remove_output_fields = ["attributes"]
    deleted_msg = {'vnf': 'delete initiated'}

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--force',
            default=False,
            action='store_true',
            help=_('Force delete VNF instance'))

    def args2body(self, parsed_args):
        body = dict()
        if parsed_args.force:
            body[self.resource] = dict()
            body[self.resource]['attributes'] = {'force': True}
        return body


class ListVNFResources(tackerV10.ListCommand):
    """List resources of a VNF like VDU, CP, etc."""

    list_columns = ['name', 'id', 'type']
    allow_names = True
    resource = _VNF

    def get_id(self):
        if self.resource:
            return self.resource.upper()

    def get_parser(self, prog_name):
        parser = super(ListVNFResources, self).get_parser(prog_name)
        if self.allow_names:
            help_str = _('ID or name of %s to look up')
        else:
            help_str = _('ID of %s to look up')
        parser.add_argument(
            'id', metavar=self.get_id(),
            help=help_str % self.resource)
        return parser

    def get_data(self, parsed_args):
        self.log.debug('get_data(%s)', parsed_args)
        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format
        if self.allow_names:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          self.resource,
                                                          parsed_args.id)
        else:
            _id = parsed_args.id

        data = self.retrieve_list_by_id(_id, parsed_args)
        self.extend_list(data, parsed_args)
        return self.setup_columns(data, parsed_args)

    def retrieve_list_by_id(self, id, parsed_args):
        """Retrieve a list of sub resources from Tacker server"""
        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format
        _extra_values = tackerV10.parse_args_to_dict(self.values_specs)
        tackerV10._merge_args(self, parsed_args, _extra_values,
                              self.values_specs)
        search_opts = self.args2search_opts(parsed_args)
        search_opts.update(_extra_values)
        if self.pagination_support:
            page_size = parsed_args.page_size
            if page_size:
                search_opts.update({'limit': page_size})
        if self.sorting_support:
            keys = parsed_args.sort_key
            if keys:
                search_opts.update({'sort_key': keys})
            dirs = parsed_args.sort_dir
            len_diff = len(keys) - len(dirs)
            if len_diff > 0:
                dirs += ['asc'] * len_diff
            elif len_diff < 0:
                dirs = dirs[:len(keys)]
            if dirs:
                search_opts.update({'sort_dir': dirs})
        obj_lister = getattr(tacker_client, "list_vnf_resources")
        data = obj_lister(id, **search_opts)
        return data.get('resources', [])


class ScaleVNF(tackerV10.TackerCommand):
    """Scale a VNF."""

    api = 'nfv-orchestration'
    resource = None
    log = None

    def get_parser(self, prog_name):
        parser = super(ScaleVNF, self).get_parser(prog_name)
        self.add_known_arguments(parser)
        return parser

    def run(self, parsed_args):
        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format
        body = self.args2body(parsed_args)
        obj_creator = getattr(tacker_client,
                              "scale_vnf")
        obj_creator(body["scale"].pop('vnf_id'), body)

    def add_known_arguments(self, parser):
        vnf_group = parser.add_mutually_exclusive_group(required=True)
        vnf_group.add_argument(
            '--vnf-id',
            help=_('VNF ID'))
        vnf_group.add_argument(
            '--vnf-name',
            help=_('VNF name'))
        parser.add_argument(
            '--scaling-policy-name',
            help=_('VNF policy name used to scale'))
        parser.add_argument(
            '--scaling-type',
            help=_('VNF scaling type, it could be either "out" or "in"'))

    def args2body(self, parsed_args):
        args = {}
        body = {"scale": args}

        if parsed_args.vnf_name:
            tacker_client = self.get_client()
            tacker_client.format = parsed_args.request_format
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vnf',
                                                          parsed_args.
                                                          vnf_name)
            parsed_args.vnf_id = _id

        args['vnf_id'] = parsed_args.vnf_id
        args['type'] = parsed_args.scaling_type
        args['policy'] = parsed_args.scaling_policy_name

        return body



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.tacker import v1_0 as tackerV10


_NS = 'ns'
_RESOURCE = 'resource'


class ListNS(tackerV10.ListCommand):
    """List NS that belong to a given tenant."""

    resource = _NS
    list_columns = ['id', 'name', 'nsd_id', 'vnf_ids', 'vnffg_ids',
                    'mgmt_ip_addresses', 'status']


class ShowNS(tackerV10.ShowCommand):
    """Show information of a given NS."""

    resource = _NS


class CreateNS(tackerV10.CreateCommand):
    """Create a NS."""

    resource = _NS
    remove_output_fields = ["attributes"]

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Set a name for the NS'))
        parser.add_argument(
            '--description',
            help=_('Set description for the NS'))
        nsd_group = parser.add_mutually_exclusive_group(required=True)
        nsd_group.add_argument(
            '--nsd-id',
            help=_('NSD ID to use as template to create NS'))
        nsd_group.add_argument(
            '--nsd-template',
            help=_('NSD file to create NS'))
        nsd_group.add_argument(
            '--nsd-name',
            help=_('NSD name to use as template to create NS'))
        vim_group = parser.add_mutually_exclusive_group()
        vim_group.add_argument(
            '--vim-id',
            help=_('VIM ID to use to create NS on the specified VIM'))
        vim_group.add_argument(
            '--vim-name',
            help=_('VIM name to use to create NS on the specified VIM'))
        parser.add_argument(
            '--vim-region-name',
            help=_('VIM Region to use to create NS on the specified VIM'))
        parser.add_argument(
            '--param-file',
            help=_('Specify parameter yaml file'))

    def args2body(self, parsed_args):
        args = {'attributes': {}}
        body = {self.resource: args}
        if parsed_args.vim_region_name:
            args.setdefault('placement_attr', {})['region_name'] = \
                parsed_args.vim_region_name

        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format
        if parsed_args.vim_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vim',
                                                          parsed_args.
                                                          vim_name)
            parsed_args.vim_id = _id
        if parsed_args.nsd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'nsd',
                                                          parsed_args.
                                                          nsd_name)
            parsed_args.nsd_id = _id
        elif parsed_args.nsd_template:
            with open(parsed_args.nsd_template) as f:
                template = f.read()
            try:
                args['nsd_template'] = yaml.load(
                    template, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not args['nsd_template']:
                raise exceptions.InvalidInput(reason='The nsd file is empty')

        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                args['attributes']['param_values'] = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'description',
                               'nsd_id', 'vim_id'])
        return body


class DeleteNS(tackerV10.DeleteCommand):
    """Delete given NS(s)."""

    resource = _NS
    deleted_msg = {'ns': 'delete initiated'}

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--force',
            default=False,
            action='store_true',
            help=_('Force delete Network Service'))

    def args2body(self, parsed_args):
        if parsed_args.force:
            body = {self.resource: {'attributes': {'force': True}}}
        else:
            body = dict()
        return body



# Copyright 2018 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import yaml

from osc_lib.command import command
from osc_lib import utils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.osc import sdk_utils
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.tacker import v1_0 as tackerV10

_attr_map = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('nsd_id', 'NSD ID', tacker_osc_utils.LIST_BOTH),
    ('vnf_ids', 'VNF IDs', tacker_osc_utils.LIST_BOTH),
    ('vnffg_ids', 'VNFFG IDs', tacker_osc_utils.LIST_BOTH),
    ('mgmt_ip_addresses', 'Mgmt Ip Addresses', tacker_osc_utils.LIST_BOTH),
    ('status', 'Status', tacker_osc_utils.LIST_BOTH),
)

_NS = 'ns'
_RESOURCE = 'resource'


def _get_columns(item):
    column_map = {
        'tenant_id': 'project_id',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class CreateNS(command.ShowOne):
    _description = _("Create a new NS")

    def get_parser(self, prog_name):
        parser = super(CreateNS, self).get_parser(prog_name)
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name for NS'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID'))
        parser.add_argument(
            '--description',
            help=_('Set description for the NS'))
        nsd_group = parser.add_mutually_exclusive_group(required=True)
        nsd_group.add_argument(
            '--nsd-id',
            help=_('NSD ID to use as template to create NS'))
        nsd_group.add_argument(
            '--nsd-template',
            help=_('NSD file to create NS'))
        nsd_group.add_argument(
            '--nsd-name',
            help=_('NSD name to use as template to create NS'))
        vim_group = parser.add_mutually_exclusive_group()
        vim_group.add_argument(
            '--vim-id',
            help=_('VIM ID to use to create NS on the specified VIM'))
        vim_group.add_argument(
            '--vim-name',
            help=_('VIM name to use to create NS on the specified VIM'))
        parser.add_argument(
            '--vim-region-name',
            help=_('VIM Region to use to create NS on the specified VIM'))
        parser.add_argument(
            '--param-file',
            help=_('Specify parameter YAML file'))
        return parser

    def args2body(self, parsed_args):
        body = {_NS: {}}
        body[_NS]['attributes'] = {}

        if parsed_args.vim_region_name:
            body[_NS].setdefault('placement_attr', {})['region_name'] = \
                parsed_args.vim_region_name

        client = self.app.client_manager.tackerclient
        if parsed_args.vim_name:
            _id = tackerV10.find_resourceid_by_name_or_id(client, 'vim',
                                                          parsed_args.vim_name)
            parsed_args.vim_id = _id
        if parsed_args.nsd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(client, 'nsd',
                                                          parsed_args.nsd_name)
            parsed_args.nsd_id = _id
        elif parsed_args.nsd_template:
            with open(parsed_args.nsd_template) as f:
                template = f.read()
                try:
                    template = yaml.load(
                        template, Loader=yaml.SafeLoader)
                except yaml.YAMLError as e:
                    raise exceptions.InvalidInput(reason=e)
                if not template:
                    raise exceptions.InvalidInput(
                        reason='The nsd file is empty')
                body[_NS]['nsd_template'] = template

        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                param_yaml = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not param_yaml:
                raise exceptions.InvalidInput(
                    reason='The parameter file is empty')
            body[_NS]['attributes'] = {'param_values': param_yaml}
        tackerV10.update_dict(parsed_args, body[_NS],
                              ['tenant_id', 'name', 'description',
                               'nsd_id', 'vim_id'])
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        ns = client.create_ns(self.args2body(parsed_args))
        display_columns, columns = _get_columns(ns[_NS])
        data = utils.get_item_properties(
            sdk_utils.DictModel(ns[_NS]),
            columns)
        lstdata = list(data)
        for index, value in enumerate(lstdata):
            if value is None:
                lstdata[index] = ''
        data = tuple(lstdata)
        return (display_columns, data)


class DeleteNS(command.Command):
    _description = _("Delete NS(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteNS, self).get_parser(prog_name)
        parser.add_argument(
            _NS,
            metavar="<NS>",
            nargs="+",
            help=_("NS(s) to delete (name or ID)")
        )
        parser.add_argument(
            '--force',
            default=False,
            action='store_true',
            help=_('Force delete Network Service')
        )
        return parser

    def args2body(self, parsed_args):
        if parsed_args.force:
            body = {_NS: {'attributes': {'force': True}}}
        else:
            body = dict()
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        failure = False
        deleted_ids = []
        failed_items = {}
        body = self.args2body(parsed_args)
        for resource_id in parsed_args.ns:
            try:
                obj = tackerV10.find_resourceid_by_name_or_id(
                    client, _NS, resource_id)
                client.delete_ns(obj, body)
                deleted_ids.append(resource_id)
            except Exception as e:
                failure = True
                failed_items[resource_id] = e
        if failure:
            msg = ''
            if deleted_ids:
                msg = (_('Successfully deleted %(resource)s(s):'
                         ' %(deleted_list)s') % {'deleted_list':
                                                 ', '.join(deleted_ids),
                                                 'resource': _NS})
            err_msg = _("\n\nUnable to delete the below"
                        " %s(s):") % _NS
            for failed_id, error in failed_items.items():
                err_msg += (_('\n Cannot delete %(failed_id)s: %(error)s')
                            % {'failed_id': failed_id,
                               'error': error})
            msg += err_msg
            raise exceptions.CommandError(message=msg)
        else:
            print((_('All specified %(resource)s(s) deleted successfully')
                   % {'resource': _NS}))
        return


class ListNS(command.Lister):
    _description = ("List (NS)s that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListNS, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        data = client.list_nss()
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map, long_listing=None)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_NS + 's']))


class ShowNS(command.ShowOne):
    _description = _("Display NS details")

    def get_parser(self, prog_name):
        parser = super(ShowNS, self).get_parser(prog_name)
        parser.add_argument(
            _NS,
            metavar="<NS>",
            help=_("NS to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _NS, parsed_args.ns)
        obj = client.show_ns(obj_id)
        display_columns, columns = _get_columns(obj[_NS])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_NS]),
            columns)
        lstdata = list(data)
        for index, value in enumerate(lstdata):
            if value is None:
                lstdata[index] = ''
        data = tuple(lstdata)
        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tackerclient.tacker import v1_0 as tackerV10
import yaml

_CLUSTER = 'cluster'
_CLUSTER_MEMBER = 'clustermember'


class ListCluster(tackerV10.ListCommand):
    """List Clusters that belong to a given tenant."""

    resource = _CLUSTER
    list_columns = ['id', 'name', 'vnfd_id', 'status', 'vip_endpoint']


class ShowCluster(tackerV10.ShowCommand):
    """Show information of a given Cluster."""

    resource = _CLUSTER


class DeleteCluster(tackerV10.DeleteCommand):
    """Delete a given Cluster."""

    resource = _CLUSTER


class CreateCluster(tackerV10.CreateCommand):
    """Create a Cluster."""

    resource = _CLUSTER

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help='Set a name for the VNF cluster')
        vnfd_group = parser.add_mutually_exclusive_group(required=True)
        vnfd_group.add_argument(
            '--vnfd-id',
            help='VNFD ID to use as template to create member VNF')
        vnfd_group.add_argument(
            '--vnfd-name',
            help='VNFD name to use as template to create member VNF')
        parser.add_argument('--policy-file',
                            help='Specify policy file for cluster',
                            required=True)
        parser.add_argument(
            '--description',
            help='Set a description for the created VNF cluster')

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format

        if parsed_args.vnfd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vnfd',
                                                          parsed_args.
                                                          vnfd_name)
            parsed_args.vnfd_id = _id
        policy_info = None
        with open(parsed_args.policy_file) as f:
            policy_info = yaml.safe_load(f.read())
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'vnfd_id', 'description'])
        if policy_info:
            body[self.resource]['policy_info'] = policy_info
        return body


class AddClusterMember(tackerV10.CreateCommand):
    """Add a new Cluster Member to given Cluster."""

    resource = _CLUSTER_MEMBER

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help='Set a name for the VNF cluster member')
        cluster_group = parser.add_mutually_exclusive_group()
        cluster_group.add_argument(
            '--cluster-id',
            help='VNFD ID to use as template to create member VNF')
        cluster_group.add_argument(
            '--cluster-name',
            help='VNFD name to use as template to create member VNF')
        vnfd_group = parser.add_mutually_exclusive_group()
        vnfd_group.add_argument(
            '--vnfd-id',
            help='Set a id for the VNFD')
        vnfd_group.add_argument(
            '--vnfd-name',
            help='Set a name for the VNFD')
        parser.add_argument(
            '--role',
            help='Set a [Active/Standby] role to cluster member',
            required=True)
        vim_group = parser.add_mutually_exclusive_group()
        vim_group.add_argument(
            '--vim-id',
            help='Set a VIM ID to deploy cluster member')
        vim_group.add_argument(
            '--vim-name',
            help='Set a VIM name to deploy cluster member')

    def args2body(self, parsed_args):
        body = {self.resource: {}}

        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format
        if parsed_args.cluster_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'cluster',
                                                          parsed_args.
                                                          cluster_name)
            parsed_args.cluster_id = _id
        if parsed_args.vnfd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vnfd',
                                                          parsed_args.
                                                          vnfd_name)
            parsed_args.vnfd_id = _id
        parsed_args.role = parsed_args.role.upper()
        if parsed_args.vim_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'vim',
                                                          parsed_args.
                                                          vim_name)
            parsed_args.vim_id = _id
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'cluster_id', 'vnfd_id',
                               'role', 'vim_id'])
        return body


class ListClusterMember(tackerV10.ListCommand):
    """List Cluster Members that belong to a given tenant."""

    resource = _CLUSTER_MEMBER

    def add_known_arguments(self, parser):
        cluster_group = parser.add_mutually_exclusive_group(required=True)
        cluster_group.add_argument(
            '--cluster-id',
            help='Set a ID for the queried cluster')
        cluster_group.add_argument(
            '--cluster-name',
            help='Set a name for the queried cluster')

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        tacker_client = self.get_client()
        tacker_client.format = parsed_args.request_format

        if parsed_args.cluster_name:
            _id = tackerV10.find_resourceid_by_name_or_id(tacker_client,
                                                          'cluster',
                                                          parsed_args.
                                                          cluster_name)
            parsed_args.cluster_id = _id

        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'cluster_id'])
        return body

    list_columns = ['id', 'name', 'cluster_id', 'role', 'vnf_id',
                    'vim_id', 'mgmt_ip_address', 'lb_member_id']


class DeleteClusterMember(tackerV10.DeleteCommand):
    """Delete a given Cluster Member."""

    resource = _CLUSTER_MEMBER


class ShowClusterMember(tackerV10.ShowCommand):
    """Show information of a given Cluster Member."""

    resource = _CLUSTER_MEMBER



# Copyright 2018 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import yaml

from osc_lib.command import command
from osc_lib import utils
from oslo_utils import encodeutils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.osc import sdk_utils
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.tacker import v1_0 as tackerV10

_attr_map = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('mgmt_ip_address', 'Mgmt Ip Address',
     tacker_osc_utils.LIST_BOTH),
    ('status', 'Status', tacker_osc_utils.LIST_BOTH),
    ('vim_id', 'VIM ID', tacker_osc_utils.LIST_BOTH),
    ('vnfd_id', 'VNFD ID', tacker_osc_utils.LIST_BOTH),
    ('tenant_id', 'Project ID', tacker_osc_utils.LIST_LONG_ONLY),
)

_attr_map_rsc = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('type', 'Type', tacker_osc_utils.LIST_BOTH),
)

_VNF = "vnf"


def _get_columns(item):
    column_map = {
        'tenant_id': 'project_id',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


def _break_string(vnf_monitoring_policy):
    count_space = 0
    monitoring_policy = "\n"
    for i in range(0, len(vnf_monitoring_policy)):
        monitoring_policy += vnf_monitoring_policy[i]
        if vnf_monitoring_policy[i] == ' ':
            count_space += 1
        if count_space == 9:
            monitoring_policy += "\n"
            count_space = 0
    return monitoring_policy


class CreateVNF(command.ShowOne):
    _description = _("Create a new VNF")

    def get_parser(self, prog_name):
        parser = super(CreateVNF, self).get_parser(prog_name)
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Set a name for the VNF'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID or project ID'))
        vnfd_group = parser.add_mutually_exclusive_group(required=True)
        vnfd_group.add_argument(
            '--vnfd-id',
            help=_('VNFD ID to use as template to create VNF'))
        vnfd_group.add_argument(
            '--vnfd-name',
            help=_('VNFD Name to use as template to create VNF'))
        vnfd_group.add_argument(
            '--vnfd-template',
            help=_("VNFD file to create VNF"))
        vim_group = parser.add_mutually_exclusive_group()
        vim_group.add_argument(
            '--vim-id',
            help=_('VIM ID to deploy VNF on specified VIM'))
        vim_group.add_argument(
            '--vim-name',
            help=_('VIM name to deploy VNF on specified VIM'))
        parser.add_argument(
            '--vim-region-name',
            help=_('VIM Region to deploy VNF on specified VIM'))
        parser.add_argument(
            '--config-file',
            help=_('YAML file with VNF configuration'))
        parser.add_argument(
            '--param-file',
            help=_('Specify parameter yaml file'))
        parser.add_argument(
            '--description',
            help=_('Set description for the VNF'))
        return parser

    def args2body(self, parsed_args):
        body = {_VNF: {}}
        body[_VNF]['attributes'] = {}

        config = None
        if parsed_args.config_file:
            with open(parsed_args.config_file) as f:
                config_yaml = f.read()
            try:
                config = yaml.load(
                    config_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
        if config:
            body[_VNF]['attributes'] = {'config': config}

        if parsed_args.vim_region_name:
            body[_VNF].setdefault('placement_attr', {})['region_name'] = \
                parsed_args.vim_region_name

        client = self.app.client_manager.tackerclient
        if parsed_args.vim_name:
            _id = tackerV10.find_resourceid_by_name_or_id(client, 'vim',
                                                          parsed_args.vim_name)
            parsed_args.vim_id = _id
        if parsed_args.vnfd_name:
            _id = tackerV10.find_resourceid_by_name_or_id(
                client, 'vnfd',
                parsed_args.vnfd_name
            )
            parsed_args.vnfd_id = _id
        elif parsed_args.vnfd_template:
            with open(parsed_args.vnfd_template) as f:
                template = f.read()
                try:
                    template = yaml.load(
                        template, Loader=yaml.SafeLoader)
                except yaml.YAMLError as e:
                    raise exceptions.InvalidInput(reason=e)
                if not template:
                    raise exceptions.InvalidInput(
                        reason='The vnfd file is empty')
                body[_VNF]['vnfd_template'] = template

        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                param_yaml = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not param_yaml:
                raise exceptions.InvalidInput(
                    reason='The parameter file is empty')
            body[_VNF]['attributes'] = {'param_values': param_yaml}
        tackerV10.update_dict(parsed_args, body[_VNF],
                              ['tenant_id', 'name', 'description',
                               'vnfd_id', 'vim_id'])
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        vnf = client.create_vnf(self.args2body(parsed_args))
        display_columns, columns = _get_columns(vnf[_VNF])
        if vnf[_VNF]['attributes'].get('monitoring_policy'):
            vnf[_VNF]['attributes']['monitoring_policy'] =\
                _break_string(vnf[_VNF]['attributes']['monitoring_policy'])
        data = utils.get_item_properties(
            sdk_utils.DictModel(vnf[_VNF]),
            columns)
        return (display_columns, data)


class DeleteVNF(command.Command):
    _description = _("Delete VNF(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteVNF, self).get_parser(prog_name)
        parser.add_argument(
            _VNF,
            metavar="<VNF>",
            nargs="+",
            help=_("VNF(s) to delete (name or ID)"))
        parser.add_argument(
            '--force',
            default=False,
            action='store_true',
            help=_('Force delete VNF instance'))
        return parser

    def args2body(self, parsed_args):
        body = dict()
        if parsed_args.force:
            body[_VNF] = dict()
            body[_VNF]['attributes'] = dict()
            body[_VNF]['attributes']['force'] = True
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        failure = False
        deleted_ids = []
        failed_items = {}
        body = self.args2body(parsed_args)
        for resource_id in parsed_args.vnf:
            try:
                obj = tackerV10.find_resourceid_by_name_or_id(
                    client, _VNF, resource_id)
                client.delete_vnf(obj, body)
                deleted_ids.append(resource_id)
            except Exception as e:
                failure = True
                failed_items[resource_id] = e
        if failure:
            msg = ''
            if deleted_ids:
                msg = (_('Successfully deleted %(resource)s(s):'
                         ' %(deleted_list)s') % {'deleted_list':
                                                 ', '.join(deleted_ids),
                                                 'resource': _VNF})
            err_msg = _("\n\nUnable to delete the below"
                        " %s(s):") % _VNF
            for failed_id, error in failed_items.items():
                err_msg += (_('\n Cannot delete %(failed_id)s: %(error)s')
                            % {'failed_id': failed_id,
                               'error': error})
            msg += err_msg
            raise exceptions.CommandError(message=msg)
        else:
            print((_('All specified %(resource)s(s) deleted successfully')
                   % {'resource': _VNF}))
        return


class ListVNF(command.Lister):
    _description = _("List VNF(s) that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListVNF, self).get_parser(prog_name)
        parser.add_argument(
            '--template-source',
            help=_("List VNF with specified template source. Available \
                   options are 'onboarded' (default), 'inline' or 'all'"),
            action='store',
            default='onboarded')
        vim_group = parser.add_mutually_exclusive_group()
        vim_group.add_argument(
            '--vim-id',
            help=_('List VNF(s) that belong to a given VIM ID'))
        vim_group.add_argument(
            '--vim-name',
            help=_('List VNF(s) that belong to a given VIM Name'))
        vnfd_group = parser.add_mutually_exclusive_group()
        vnfd_group.add_argument(
            '--vnfd-id',
            help=_('List VNF(s) that belong to a given VNFD ID'))
        vnfd_group.add_argument(
            '--vnfd-name',
            help=_('List VNF(s) that belong to a given VNFD Name'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID or project ID'))
        parser.add_argument(
            '--long',
            action='store_true',
            help=_('List additional fields in output'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        _params = {}
        if parsed_args.vim_id:
            _params['vim_id'] = parsed_args.vim_id
        if parsed_args.vim_name:
            vim_id = tackerV10.find_resourceid_by_name_or_id(
                client, 'vim', parsed_args.vim_name
            )
            _params['vim_id'] = vim_id
        if parsed_args.vnfd_id:
            _params['vnfd_id'] = parsed_args.vnfd_id
        if parsed_args.vnfd_name:
            vim_id = tackerV10.find_resourceid_by_name_or_id(
                client, 'vnfd', parsed_args.vnfd_name
            )
            _params['vnfd_id'] = vim_id
        if parsed_args.tenant_id:
            _params['tenant_id'] = parsed_args.tenant_id
        data = client.list_vnfs(**_params)
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_VNF + 's']))


class ShowVNF(command.ShowOne):
    _description = _("Display VNF details")

    def get_parser(self, prog_name):
        parser = super(ShowVNF, self).get_parser(prog_name)
        parser.add_argument(
            _VNF,
            metavar="<VNF>",
            help=_("VNF to display (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNF, parsed_args.vnf)
        obj = client.show_vnf(obj_id)
        if obj[_VNF]['attributes'].get('monitoring_policy'):
            obj[_VNF]['attributes']['monitoring_policy'] =\
                _break_string(obj[_VNF]['attributes']['monitoring_policy'])
        display_columns, columns = _get_columns(obj[_VNF])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_VNF]),
            columns)
        return (display_columns, data)


class ListVNFResources(command.Lister):
    _description = _("List resources of a VNF like VDU, CP, etc.")

    def get_parser(self, prog_name):
        parser = super(ListVNFResources, self).get_parser(prog_name)
        parser.add_argument(
            _VNF,
            metavar="<VNF>",
            help=_("VNF to display (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNF, parsed_args.vnf)
        data = client.list_vnf_resources(obj_id)
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map_rsc, long_listing=None)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data['resources']))


class UpdateVNF(command.ShowOne):
    _description = _("Update a given VNF.")

    def get_parser(self, prog_name):
        parser = super(UpdateVNF, self).get_parser(prog_name)
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument(
            '--config-file',
            help=_('YAML file with VNF configuration'))
        group.add_argument(
            '--config',
            help=_('YAML data with VNF configuration'))
        group.add_argument(
            '--param-file',
            help=_('YAML file with VNF parameter'))
        parser.add_argument(
            _VNF,
            metavar="<VNF>",
            help=_("VNF to update (name or ID)"))
        return parser

    def args2body(self, parsed_args):
        body = {_VNF: {}}
        body[_VNF]['attributes'] = {}

        config = None
        if parsed_args.config_file:
            with open(parsed_args.config_file) as f:
                config_yaml = f.read()
            try:
                config = yaml.load(config_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not config:
                raise exceptions.InvalidInput(
                    reason='The config file is empty')
        if parsed_args.config:
            decoded_config = encodeutils.safe_decode(parsed_args.config)
            try:
                config = yaml.load(decoded_config, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not config:
                raise exceptions.InvalidInput(
                    reason='The parameter is empty')
        if config:
            body[_VNF]['attributes'] = {'config': config}
        if parsed_args.param_file:
            with open(parsed_args.param_file) as f:
                param_yaml = f.read()
            try:
                param = yaml.load(
                    param_yaml, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not param:
                raise exceptions.InvalidInput(
                    reason='The parameter file is empty')
            body[_VNF]['attributes'] = {'param_values': param}
        tackerV10.update_dict(parsed_args, body[_VNF], ['tenant_id'])
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNF, parsed_args.vnf)
        vnf = client.update_vnf(obj_id, self.args2body(parsed_args))
        if vnf[_VNF]['attributes'].get('monitoring_policy'):
            vnf[_VNF]['attributes']['monitoring_policy'] =\
                _break_string(vnf[_VNF]['attributes']['monitoring_policy'])
        display_columns, columns = _get_columns(vnf[_VNF])
        data = utils.get_item_properties(
            sdk_utils.DictModel(vnf[_VNF]),
            columns)
        return (display_columns, data)


class ScaleVNF(command.Command):
    _description = _("Scale a VNF.")

    def get_parser(self, prog_name):
        parser = super(ScaleVNF, self).get_parser(prog_name)
        parser.add_argument(
            '--scaling-policy-name',
            help=_('VNF policy name used to scale'))
        parser.add_argument(
            '--scaling-type',
            help=_('VNF scaling type, it could be either "out" or "in"'))
        parser.add_argument(
            _VNF,
            metavar="<VNF>",
            help=_("VNF to scale (name or ID)"))
        return parser

    def args2body(self, parsed_args):
        args = {}
        body = {"scale": args}
        args['type'] = parsed_args.scaling_type
        args['policy'] = parsed_args.scaling_policy_name
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNF, parsed_args.vnf)
        client.scale_vnf(obj_id, self.args2body(parsed_args))
        return
